// In ModBus Communication we have one master and one or more slave.
// Master send a request signal for slave on bus. 
// Slave answer sending data to master.

// It's strongly recommended to try this sketch with QModBus


/******************POSSIBILI CONFIGURAZIONI DEI PIN*******************

  DIGITAL IN: 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, A0, A1, A2, A3, A4, A5          

  DIGITAL OUT: 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, A0, A1, A2, A3, A4, A5          

  ANALOG IN: A0, A1, A2, A3, A4, A5   

  ANALOG OUT: 3, 5, 6, 10, 11 

// Can't use PIN 0 and 1 (dedicated to serial communication
// PIN 13 can have some problem due to Builtin-LED
// A0, A1, can be used also as digital input or output
*****/

#include <Arduino.h>
#include "Modbus.h"
#include "ModbusSerial.h"

//ModBus Port information
#define BAUD        115200 

//Baud Rate
#define ID          1 // id CHE IDENTIFICA ARDUINO NELLA RETE
#define TXPIN       -1 //Don't change it

//Define the number of registers, inputs and coils to be created
#define NUM_COILS               2       // NUMERO DI PIN OUTPUT DIGITALI
#define NUM_DISCRETE_INPUT      2       // NUMERO DI PIN INPUT DIGITALI
#define NUM_HOLDING_REGISTERS   2       // NUMERO DI OUTPUT ANALOGICI
#define NUM_INPUT_REGISTERS     2       // NUMERO DI INPUT ANALOGICI

//Create the I/O pin masks
uint8_t pinMask_DOUT[] = {12, 4};        //%IX0.0   %IX0.1   PORTE DEI LED
uint8_t pinMask_DIN[] = {7, 8};        //%QX0.0   %QX0.1   PORTE DEI PULSANTI IN PULL-DOWN
uint8_t pinMask_AOUT[] = {5, 3};        //%IW1     %IW2     PORTE USCITE ANALOGICHE
uint8_t pinMask_AIN[] = {A0, A1};       //%QW1     %QW2     PORTE INGRESSI ANALOGICI

//Note that we have to connect Button in pull-down or pull-up configuration

//QModBus Test

// Read Digital  (Value 0 or 1)    ==>   Read Discrete Inputs    (function 0x02)
// Write Digital (Value 0 or 1)    ==>   Write Coil              (function 0x05)
// Write Analog (From 0 to 255)    ==>   Write Single Register   (function 0x06)     
// Read Analog  (From 0 to 1023)   ==>   Read Input Register     (function 0x04)

// We can read more than one digital input with the same function (0x02)
// We can read more than one analog input with the same function (0x04)
// We can write multiple digital output with "Write multiple coils" (function 0x0f)
// We can write multiple analog output with "Write multiple register" (function 0x10)

/**
Coding part of sketch
*****/
//Modbus Object
ModbusSerial modbus;

void configurePins()
{
  for (int i = 0; i < NUM_DISCRETE_INPUT; i++)
  {
    pinMode(pinMask_DIN[i], INPUT);
  }

  for (int i = 0; i < NUM_INPUT_REGISTERS; i++)
  {
    pinMode(pinMask_AIN[i], INPUT);
  }

  for (int i = 0; i < NUM_COILS; i++)
  {
    pinMode(pinMask_DOUT[i], OUTPUT);
  }

  for (int i = 0; i < NUM_HOLDING_REGISTERS; i++)
  {
    pinMode(pinMask_AOUT[i], OUTPUT);
  }
}

void setup()
{

  //Setup board I/O
  configurePins();

  //Config Modbus Serial (port, speed, rs485 tx pin)
  modbus.config(&Serial, BAUD, TXPIN);

  //Set the Slave ID
  modbus.setSlaveId(ID);

  //Add all modbus registers
  for (int i = 0; i < NUM_DISCRETE_INPUT; ++i)
  {
    modbus.addIsts(i);
  }
  for (int i = 0; i < NUM_INPUT_REGISTERS; ++i)
  {
    modbus.addIreg(i);
  }
  for (int i = 0; i < NUM_COILS; ++i)
  {
    modbus.addCoil(i);
  }
  for (int i = 0; i < NUM_HOLDING_REGISTERS; ++i)
  {
    modbus.addHreg(i);
  }
}

void loop()
{
    digitalWrite(8,LOW);
  digitalWrite(12,LOW);
  //Run the main modbus task
  modbus.task();

  //Update modbus registers
  for (int i = 0; i < NUM_DISCRETE_INPUT; ++i)
  {
    modbus.Ists(i, digitalRead(pinMask_DIN[i]));
  }
  for (int i = 0; i < NUM_INPUT_REGISTERS; ++i)
  {
    modbus.Ireg(i, (analogRead(pinMask_AIN[i]) ));
    //modbus.Ireg(i, (analogRead(pinMask_AIN[i]) * 64))
  }
  for (int i = 0; i < NUM_COILS; ++i)
  {
    digitalWrite(pinMask_DOUT[i], modbus.Coil(i));
  }
  for (int i = 0; i < NUM_HOLDING_REGISTERS; ++i)
  {
    analogWrite(pinMask_AOUT[i], (modbus.Hreg(i) / 4));
    //   analogWrite(pinMask_AOUT[i], (modbus.Hreg(i) / 256));
  }
}
